@@ -1,12 +1,11 @@
-/*
- * App.m
+/* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2004 by vhf interservice GmbH
+ * Copyright (C) 1995-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2004-04-20
+ * modified: 2005-01-04
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -55,7 +54,7 @@
 @implementation App
 
 /*
- * modified: 2004-04-20
+ * modified: 2004-08-04
  * Initializes the defaults.
  */
 + (void)initialize
@@ -65,26 +64,29 @@
 
     /* General Preferences defaults */
 #ifdef __APPLE__
-    [registrationDict setObject:@"NO" forKey:@"doCaching"];
+    [registrationDict setObject:@"NO"  forKey:@"doCaching"];
 #else
     [registrationDict setObject:@"YES" forKey:@"doCaching"];
 #endif
-    [registrationDict setObject:@"0" forKey:@"unit"];
-    [registrationDict setObject:@"NO" forKey:@"removeBackups"];
-    [registrationDict setObject:@"NO" forKey:@"expertMode"];
-    [registrationDict setObject:@"0" forKey:@"snap"];
-    [registrationDict setObject:@"." forKey:@"NSDecimalSeparator"];
+    [registrationDict setObject:@"0"   forKey:@"unit"];
+    [registrationDict setObject:@"NO"  forKey:@"removeBackups"];
+    [registrationDict setObject:@"NO"  forKey:@"expertMode"];
+    [registrationDict setObject:@"0"   forKey:@"snap"];
+    [registrationDict setObject:@"0"   forKey:@"lineWidth"];
+    [registrationDict setObject:@"20"  forKey:@"cacheLimit"];
 
     /* Import preferences defaults */
-    [registrationDict removeAllObjects];
     [registrationDict setObject:@"hpgl_8Pen" forKey:@"hpglParmsFileName"];
-    [registrationDict setObject:@"gerber" forKey:@"gerberParmsFileName"];
-    [registrationDict setObject:@"" forKey:@"dinParmsFileName"];
-    [registrationDict setObject:@"25.4" forKey:@"dxfRes"];
-    [registrationDict setObject:@"NO" forKey:@"psFlattenText"];
-    [registrationDict setObject:@"NO" forKey:@"psPreferArcs"];
-    [registrationDict setObject:@"NO" forKey:@"colorToLayer"];
-    [registrationDict setObject:@"NO" forKey:@"fillObjects"];
+    [registrationDict setObject:@"gerber"    forKey:@"gerberParmsFileName"];
+    [registrationDict setObject:@""          forKey:@"dinParmsFileName"];
+    [registrationDict setObject:@"25.4"      forKey:@"dxfRes"];
+    [registrationDict setObject:@"NO"        forKey:@"psFlattenText"];
+    [registrationDict setObject:@"NO"        forKey:@"psPreferArcs"];
+    [registrationDict setObject:@"NO"        forKey:@"colorToLayer"];
+    [registrationDict setObject:@"NO"        forKey:@"fillObjects"];
+
+    /* Export preferences defaults */
+    [registrationDict setObject:@"NO"        forKey:@"exportFlattenText"];
 
     [[NSUserDefaults standardUserDefaults] registerDefaults:registrationDict];
 }
@@ -105,11 +107,16 @@
  * Directory where we are currently "working."
  */
 - (NSString*)currentDirectory
-{   NSString	*cdir = [[self currentDocument] directory];
+{   NSString	*cdir = [[self currentDocument] directory], *path;
 
-    return (cdir && ![cdir isEqual:@""]) ? cdir
-                                         : (haveOpenedDocument ? [((NSOpenPanel *)[NSOpenPanel openPanel]) directory]
-                                                               : vhfHomeDirectory());
+    path = (cdir && [cdir length]) ? cdir
+                                   : (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
+                                                         : vhfHomeDirectory());
+    if (!path)
+        return vhfHomeDirectory();
+    if (!path)	// Apple Check to find nil directories
+        NSLog(@"[App currentDirectory]: path == nil, NSHomeDirectory() = '%@'", NSHomeDirectory());
+    return path;
 }
 
 /* set the current document without regard to the active window
@@ -161,7 +168,7 @@
 }
 
 /* created:  2002-07-18
- * modified: 2003-04-05
+ * modified: 2005-01-04
  */
 - (void)loadModules
 {   NSBundle		*mainBundle = [NSBundle mainBundle], *bundle;
@@ -169,18 +176,25 @@
     NSString		*path;
     int			i, j;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
+    BOOL		camModuleLoaded = NO;
 
-    /* load modules from application bundle */
+    /* load modules from application bundle
+     * FIXME: This should be integrated below (case 0)
+     *        Each module should give info about other modules needed first
+     *        Each module should give info about modules to be in conflict
+     *        Available modules should be selectable by switches in preferences
+     */
     path = [mainBundle pathForResource:@"CAM" ofType:@"bundle"];
     if ( (bundle = [NSBundle bundleWithPath:path]) )
     {
         bundleClass = [bundle principalClass];	// controller
         [modules addObject:bundle];		// our loaded modules
         [bundleClass instance];			// create instance
+        camModuleLoaded = YES;
     }
 
     /* load additional modules */
-    for (i=0; i<4; i++)
+    for (i=0; i<5; i++)
     {   NSArray	*files;
 
         switch (i)
@@ -190,12 +204,16 @@
             case 1: path = [localLibrary() stringByAppendingPathComponent:@"Bundles"]; break;
             case 2: path = userBundlePath(); break;
             case 3: path = localBundlePath(); break;
+            case 4: path = systemBundlePath(); break;
         }
 
-        files = [fileManager directoryContentsAtPath:path];
+        /* we load the bundles in alphabetic order */
+        files = [[fileManager directoryContentsAtPath:path] sortedArrayUsingSelector:@selector(compare:)];
         for (j=0; j<(int)[files count]; j++)
         {   NSString	*file = [files objectAtIndex:j];
 
+            if ([file hasPrefix:@"Cut."] && camModuleLoaded)	// either CAM or Cut, not both
+                continue;
             if ( [file hasSuffix:@".bundle"] &&
                  (bundle = [NSBundle bundleWithPath:[path stringByAppendingPathComponent:file]]) )
             {
@@ -959,7 +977,7 @@
 }
 
 /*
- * 
+ * modified: 2004-12-03
  */
 - (void)setCurrent2DTool:sender
 {   id		cursor;
@@ -982,8 +1000,8 @@
             case TOOL2D_ROTATE:		// rotate
                 if (!rotateCursor)
                 {
-                    rotateCursor = [[NSCursor alloc] initWithImage:[NSImage imageNamed:@"cursorRotate2D.tiff"]
-                                                           hotSpot:NSMakePoint(7, 7)];
+                    rotateCursor = [[NSCursor alloc] initWithImage:[NSImage imageNamed:@"cursorRotate.tiff"]
+                                                           hotSpot:NSMakePoint(5, 2)]; // was 7, 7
                 }
                 cursor = rotateCursor;
                 break;
