@@ -1,11 +1,11 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2003 by vhf interservice GmbH
+ * Copyright (C) 1996-2005 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2003-06-26
+ * Modified: 2005-01-11
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -290,7 +290,21 @@ static NSMutableArray *zoneList = nil;
                 [newDocument->window setFrameOrigin:NSMakePoint(NSMinX(winLoc) + XOFFSET, NSMinY(winLoc) + YOFFSET)];
             }
 
-            [newDocument->window setFrameFromString:[plist objectForKey:@"WindowFrame"]];	// window frame
+            [newDocument->window setFrameFromString:[plist objectForKey:@"WindowFrame"]];
+            /* GNUSTEP: make sure window is not bigger than necessary
+             *          (the window frame doesn't include title bar and resize bar)
+             */
+#ifdef GNUSTEP_BASE_VERSION
+            {   NSRect	r = rectFromPropertyList([plist objectForKey:@"ViewSize"]);
+
+                r.size.height += [newDocument->window coordBoxSize].height;
+                r.size.height += [NSScroller scrollerWidth] + 1.0;
+                r.size.width  += [NSScroller scrollerWidth] + 2.0;
+                [newDocument->window setContentSize:r.size];
+                /* we call setFrame again to avoid X-Windows error. Why is this necessary ??? */
+                [newDocument->window setFrameFromString:[newDocument->window stringWithSavedFrame]];
+            }
+#endif
 
             /* workaround: something (NSWindow according documentation) doesn't work with size>10000.0 */
             fRect = rectFromPropertyList([plist objectForKey:@"ViewSize"]);
@@ -750,47 +764,55 @@ static NSMutableArray *zoneList = nil;
  * will not be changed (unless one is currently not set, in which case
  * a default name will be used).
  */
-- (void)setName:(NSString *)newName andDirectory:(NSString *)newDirectory
+- (void)setName:(NSString*)newName andDirectory:(NSString*)newDirectory
 {   static int	untitledCount = 0;
+    NSString	*fileName;
 
-    if ((newName && ![newName isEqual:@""]) || !name)
+    if ((newName && [newName length]) || !name)
     {
- 	if (!newName || [newName isEqual:@""])
+ 	if (!newName || ![newName length])
         {
 	    newName = UNTITLED_STRING;
-	    // Should probably keep count of current UNTITLED documents.  When that goes to 0, set untitledCount = 0.
 	    if (untitledCount)
-		newName = [newName stringByAppendingString:[NSString stringWithFormat:@"%d", untitledCount]];
+		newName = [newName stringByAppendingFormat:@"%d", untitledCount];
 	    untitledCount++;
 	}
 	if (![newName isEqual:name])
         {
 	    [name autorelease];
-	    name = [newName copyWithZone:(NSZone *)[self zone]];
+	    name = [newName copyWithZone:(NSZone*)[self zone]];
 	}
     }
 
-    if ((newDirectory && ![newDirectory isEqual:@""]) || !directory)
+    if ((newDirectory && [newDirectory length]) || !directory)
     {
- 	if (!newDirectory || [newDirectory isEqual:@""])
+ 	if (!newDirectory || ![newDirectory length])
 	    newDirectory = [(App*)NSApp currentDirectory];
 	if (![newDirectory isEqual:directory])
         {
 	    [directory autorelease];
-	    directory = [newDirectory copyWithZone:(NSZone *)[self zone]];
+	    directory = [newDirectory copyWithZone:(NSZone*)[self zone]];
 	}
     }
 
-    [window setTitleWithRepresentedFilename:[self filename]];
-    [window setMiniwindowTitle:[self filename]];
-    NSSetZoneName((NSZone *)[self zone], [self filename]); 
+    if (!name)
+        NSLog(@"-setName: name == nil, newName = '%@'", newName);
+    if (!directory)
+        NSLog(@"-setName: directory == nil, newDirectory = '%@'", newDirectory);
+
+    if ( (fileName = [self filename]) )
+    {
+        [window setTitleWithRepresentedFilename:fileName];
+        [window setMiniwindowTitle:fileName];
+        NSSetZoneName((NSZone*)[self zone], fileName);
+    }
 }
 
-- (BOOL)setName:(NSString *)file
 /*
  * If file is a full path name, then both the name and directory of the
  * document is updated appropriately, otherwise, only the name is changed.
  */
+- (BOOL)setName:(NSString *)file
 {
     if (file)
     {
