@@ -1,10 +1,10 @@
 /* VImage.m
  *
- * Copyright (C) 1996-2003 by vhf interservice GmbH
+ * Copyright (C) 1996-2004 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-03-22
- * modified: 2003-06-26
+ * modified: 2004-10-06
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -42,12 +42,11 @@ static int currentGraphicIdentifier = 1;
 
 @implementation VImage
 
-static BOOL checkImage(NSImage *anImage)
-/*
- * Locking focus on an NSImage forces it to draw and thus verifies
+/* Locking focus on an NSImage forces it to draw and thus verifies
  * whether there are any PostScript or TIFF errors in the source of
- * the image.  lockFocus returns YES only if there are no errors.
+ * the image. lockFocus returns YES only if there are no errors.
  */
+static BOOL checkImage(NSImage *anImage)
 {
     if ([anImage isValid])
     { 
@@ -58,14 +57,115 @@ static BOOL checkImage(NSImage *anImage)
     return NO;
 }
 
+#if 0	// not used yet, and not many of the methods available for GNUstep, yet
+//#if defined(GNUSTEP_BASE_VERSION) || defined(__APPLE__)
+
+static NSBitmapImageFileType fileTypeFromVImageType(VImageFileType fileType)
+{
+    switch (fileType)
+    {
+        case VImageTIFF_None:
+        case VImageTIFF_LZW:
+        case VImageTIFF_PackBits: return NSTIFFFileType;
+        case VImageBMP:           return NSBMPFileType;
+        case VImageGIF:           return NSGIFFileType;
+        case VImageJPEG:          return NSJPEGFileType;
+        case VImagePNG:           return NSPNGFileType;
+    }
+    return NSTIFFFileType;
+}
+static NSDictionary *propertiesForFileType(VImageFileType fileType, float compressionFactor)
+{   NSMutableDictionary	*dict = [NSMutableDictionary dictionary];
+
+    switch (fileType)
+    {
+        case VImageTIFF_None:
+            [dict setObject:[NSNumber numberWithInt:NSTIFFCompressionNone] forKey:NSImageCompressionMethod];
+            break;
+        case VImageTIFF_LZW:
+            [dict setObject:[NSNumber numberWithInt:NSTIFFCompressionLZW] forKey:NSImageCompressionMethod];
+            break;
+        case VImageTIFF_PackBits:
+            [dict setObject:[NSNumber numberWithInt:NSTIFFCompressionPackBits] forKey:NSImageCompressionMethod];
+            break;
+        case VImageBMP:
+            break;
+        case VImageGIF:
+            break;
+        case VImageJPEG:
+            [dict setObject:[NSNumber numberWithFloat:compressionFactor] forKey:NSImageCompressionFactor];
+            break;
+        case VImagePNG:
+            break;
+    }
+    return dict;
+}
+#endif
+
+static NSString *fileExtensionFromFileType(VImageFileType fileType)
+{
+    switch (fileType)
+    {
+        case VImageTIFF_None:
+        case VImageTIFF_LZW:
+        case VImageTIFF_PackBits: return @"tiff";
+        case VImageBMP:           return @"bmp";
+        case VImageGIF:           return @"gif";
+        case VImageJPEG:          return @"jpg";
+        case VImagePNG:           return @"png";
+        //default:                  return @"tiff";
+    }
+    return nil;
+}
+static NSTIFFCompression tiffCompressionFromVImageType(VImageFileType fileType)
+{
+    switch (fileType)
+    {
+        case VImageTIFF_LZW:      return NSTIFFCompressionLZW;
+        case VImageTIFF_PackBits: return NSTIFFCompressionPackBits;
+        default:                  return NSTIFFCompressionNone;
+    }
+    return NSTIFFCompressionNone;
+}
+static VImageFileType fileTypeForTIFFCompression(NSTIFFCompression tiffCompression)
+{
+    switch (tiffCompression)
+    {
+        case NSTIFFCompressionLZW:      return VImageTIFF_LZW;
+        case NSTIFFCompressionPackBits: return VImageTIFF_PackBits;
+        default:                        return VImageTIFF_None;
+    }
+    return VImageTIFF_None;
+}
+
 /* This sets the class version so that we can compatibly read old objects out of an archive.
  */
 + (void)initialize
 {
-    [VImage setVersion:3];
+    [VImage setVersion:4];
     return;
 }
 
+/* return accepted file type for platform */
++ (BOOL)isAcceptedFileType:(VImageFileType)type
+{
+    switch (type)
+    {
+#if defined(GNUSTEP_BASE_VERSION)
+        case VImageTIFF_LZW:      return NO;
+        default:                  return YES;
+#elif defined(__APPLE__)
+        default:                  return YES;
+#else
+        case VImageTIFF_None:
+        case VImageTIFF_LZW:
+        case VImageTIFF_PackBits: return YES;
+        default:                  return NO;
+#endif
+    }
+    return YES;
+}
+
 /* initialize
  */
 - init
@@ -142,7 +242,7 @@ static BOOL checkImage(NSImage *anImage)
     [nImage setOrigin:origin];
     //[nImage setParameter]; // init call it
     [nImage setThumbnail:thumbnail];
-    [nImage setCompressionType:compressionType];
+    [nImage setFileType:fileType];
     [nImage setCompressionFactor:compressionFactor];
     if (clipPath)
         [nImage join:clipPath];
@@ -156,13 +256,14 @@ static BOOL checkImage(NSImage *anImage)
 #ifdef USE_VHF_CLIPPING
     [clipImage release]; clipImage = nil;
 #endif
-    image = nImage;
+    image = [nImage retain];
+    originalSize = [image size];
     dirty = YES;
 }
 - (NSImage*)image	{ return image; }
 
 /*
- * created: 25.09.95
+ * created: 1995-09-25
  * purpose: initializes all the stuff needed after a -read:
  */
 - (void)setParameter
@@ -173,8 +274,8 @@ static BOOL checkImage(NSImage *anImage)
     thumbnail = NO;
     imageFile = [[NSString stringWithFormat:@"Image%d.tiff", identifier] retain];
     sourcePath = nil;
-    compressionType = NSTIFFCompressionNone;
-    compressionFactor = 1.0;
+    fileType = VImageTIFF_LZW;
+    compressionFactor = 0.1;
     compressionDirty = NO;
 }
 
@@ -359,8 +460,8 @@ static BOOL checkImage(NSImage *anImage)
 }
 - (BOOL)thumbnail			{ return thumbnail; }
 
-- (void)setCompressionType:(int)type	{ compressionType = type; compressionDirty = YES; }
-- (int)compressionType;			{ return compressionType; }
+- (void)setFileType:(VImageFileType)type	{ fileType = type; compressionDirty = YES; }
+- (VImageFileType)fileType;			{ return fileType; }
 - (void)setCompressionFactor:(float)f;	{ compressionFactor = f; compressionDirty = YES; }
 - (float)compressionFactor;		{ return compressionFactor; }
 
@@ -1000,6 +1101,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             }
         }
 
+        /* Color separation
+         */
         if (!VHFIsDrawingToScreen() && [principal separationColor])
         {
             if (!(sepImage = [self separateImageWithColor:[principal separationColor]]))
@@ -1037,6 +1140,9 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             if (currentSize.width != bounds.size.width || currentSize.height != bounds.size.height)
                 [sepImage setSize:bounds.size];
         }
+
+        /* clip and composite image
+         */
         if (clipPath) // clip with clipPath gsave
         {
 #ifdef USE_VHF_CLIPPING
@@ -1077,7 +1183,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 NSPoint	currentPoint = NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
 
                 for (i=0; i<cnt; i++)
-                    currentPoint = [[[clipPath list] objectAtIndex:i] appendToBezierPath:bPath currentPoint:currentPoint];
+                    currentPoint = [[[clipPath list] objectAtIndex:i] appendToBezierPath:bPath
+                                                                            currentPoint:currentPoint];
             }
             else
             {   NSPoint	currentPoint = NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
@@ -1094,12 +1201,21 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
             NSRectFill(bounds);
         }*/
 
+        /* display or print
+         */
 #ifndef __APPLE__	// Apple has antialiasing
         if (!thumbnail && VHFIsDrawingToScreen() && [self compositeAntialiased:scale toPoint:p])
             return;
 #endif
         if ([principal cache] && !sepImage) // not for colorseparation
-            [((thumbImage && VHFIsDrawingToScreen())?thumbImage:image) compositeToPoint:p operation:NSCompositeSourceOver];
+        {   NSImage	*img = (thumbImage && VHFIsDrawingToScreen()) ? thumbImage : image;
+
+            PSgsave();	// needed for GNUstep
+            [img compositeToPoint:p operation:NSCompositeSourceOver];
+            PSgrestore();
+        }
+        /* Color separation
+         */
         else if ( (sepImage /*&& !VHFIsDrawingToScreen() && [principal separationColor]*/) ||
                   (bounds.size.width < 10000.0 && bounds.size.height < 10000.0)) // 10000 86000000
         {
@@ -1115,18 +1231,19 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 p = rect.origin;
                 rect.origin.x = (rect.origin.x - pOld.x)*scale;
                 rect.origin.y = (rect.origin.y - pOld.y)*scale;
-                rect.size.width *= scale; // important
+                rect.size.width  *= scale; // important
                 rect.size.height *= scale;
 
                 if (sepImage /*&& !VHFIsDrawingToScreen() && [principal separationColor]*/)
-                {    [sepImage compositeToPoint:p fromRect:rect operation:NSCompositeSourceOver];
+                {   [sepImage compositeToPoint:p fromRect:rect operation:NSCompositeSourceOver];
                     [sepImage release];
                 }
                 else
-                    [((thumbImage && VHFIsDrawingToScreen())?thumbImage:image) compositeToPoint:p fromRect:rect 	operation:NSCompositeSourceOver];
+                    [((thumbImage && VHFIsDrawingToScreen()) ? thumbImage : image)
+                     compositeToPoint:p fromRect:rect operation:NSCompositeSourceOver];
             }
         }
-        /* work around
+        /* display huge images with work around
          * openstep doesnt work good if we draw more than 10 000 000 pixels
          * we draw here only the visible part
          */
@@ -1144,7 +1261,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 [self compositeInRect:rect toPoint:p withScale:scale];
             }
         }
-        /* release image for printing */
+
+        /* release print image */
         if (releaseImage)
         {   [image release];
             image = nil;
@@ -1491,8 +1609,9 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
     NSArray		*imageReps;
     NSBitmapImageRep	*bmImageRep = nil;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
+    NSData		*imageData;
 
-    if (!image) // load image
+    if (!image)	// load image
     {	NSString	*backupFilename = [directory stringByAppendingString:@"~"];
         NSArray		*reps = nil;
         NSString	*backupPath, *backupSourcePath;
@@ -1532,11 +1651,8 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
         NSBitmapImageRep	*cmykbitmap;
         NSString		*cmykFilename = nil;
 
-        /* no LZW available on GNUstep ! */
-#ifdef GNUSTEP_BASE_VERSION
-        if (compressionType == NSTIFFCompressionLZW)
-            compressionType = NSTIFFCompressionJPEG;
-#endif
+        if (![VImage isAcceptedFileType:fileType])
+            fileType = VImageTIFF_None;	// default
 
         cmykFilename = [NSString stringWithFormat:@"%@/%@_cmyk.tiff", [directory stringByDeletingLastPathComponent], [imageFile stringByDeletingPathExtension]];
 
@@ -1557,9 +1673,9 @@ void antialiasColorFromBits(unsigned char **data, int byteY, int px, int bps, in
                 image = nil;
             }
             /* save to file imageFile_cmyk in sourcePath */ // [imageFile lastPathComponent] ??
-            [[image TIFFRepresentationUsingCompression:compressionType factor:compressionFactor]
-                                                                      writeToFile:cmykFilename
-                                                                       atomically:NO];
+            imageData = [image TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
+                                                           factor:0.0];
+            [imageData writeToFile:cmykFilename atomically:NO];
             return YES;
         }
         if ([bmImageRep colorSpaceName] != NSCalibratedRGBColorSpace)
@@ -1673,9 +1789,9 @@ y = Max(0, y-20);*/
         }
 
         /* save to file imageFile_cmyk in sourcePath */ // [imageFile lastPathComponent] ??
-        [[cmykImage TIFFRepresentationUsingCompression:compressionType factor:compressionFactor]
-                                                                  writeToFile:cmykFilename
-                                                                   atomically:NO];
+        imageData = [cmykImage TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
+                                                           factor:0.0];
+        [imageData writeToFile:cmykFilename atomically:NO];
         return YES;
     }
     if (thumbnail && thumbImage)
@@ -1694,11 +1810,8 @@ y = Max(0, y-20);*/
     NSString		*thFilename = [directory stringByAppendingPathComponent:[imageFile lastPathComponent]];
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
-        /* no LZW available on GNUstep ! */
-#ifdef GNUSTEP_BASE_VERSION
-        if (compressionType == NSTIFFCompressionLZW)
-            compressionType = NSTIFFCompressionJPEG;
-#endif
+    if (![VImage isAcceptedFileType:fileType])
+        fileType = VImageTIFF_None;	// default
 
     if (thumbnail && thumbImage)
     {   NSString	*str = [NSString stringWithFormat:@"%@_.%@", [imageFile stringByDeletingPathExtension],
@@ -1708,7 +1821,7 @@ y = Max(0, y-20);*/
             [[thumbImage TIFFRepresentationUsingCompression:NSTIFFCompressionNone factor:0.0]
              writeToFile:thFilename atomically:NO];
     }
-    if (compressionDirty && !image) // load image and save new (compressionType perhaps changed)
+    if (compressionDirty && !image) // load image and save new (fileType has changed)
     {	NSString	*backupFilename = [directory stringByAppendingString:@"~"];
         NSArray		*reps = nil;
         NSString	*backupPath, *backupSourcePath;
@@ -1750,14 +1863,22 @@ y = Max(0, y-20);*/
                 [sourcePath release];
             sourcePath = [filename retain];
         }
-        /*else we need a message */
+        /* else we need a message */
         else
-            NSRunAlertPanel(SAVE_TITLE, CANT_SAVE,nil ,nil , nil);
+            NSRunAlertPanel(SAVE_TITLE, CANT_SAVE, nil, nil, nil);
     }
     else if (image && ![fileManager fileExistsAtPath:filename])
-    {
-        [[image TIFFRepresentationUsingCompression:compressionType factor:compressionFactor] writeToFile:filename
-                                                                                              atomically:NO];
+    {   NSData	*data;
+
+//#if !defined(GNUSTEP_BASE_VERSION) && !defined(__APPLE__)	// OpenStep 4.2
+        data = [image TIFFRepresentationUsingCompression:tiffCompressionFromVImageType(fileType)
+                                                  factor:0.0];
+/*#else
+        data = [NSBitmapImageRep representationOfImageRepsInArray:[image representations]
+                                                        usingType:fileTypeFromVImageType(fileType)
+                                 properties:propertiesForFileType(fileType, compressionFactor)];
+#endif*/
+        [data writeToFile:filename atomically:NO];
         if (sourcePath)
             [sourcePath release];
         sourcePath = [filename retain];
@@ -1781,7 +1902,7 @@ y = Max(0, y-20);*/
     [aCoder encodeValuesOfObjCTypes:"@", &thumbImage];
     [aCoder encodeValuesOfObjCTypes:"ci", &relief, &reliefType];
     [aCoder encodeValuesOfObjCTypes:"@", &clipPath];
-    [aCoder encodeValuesOfObjCTypes:"cif", &thumbnail, &compressionType , &compressionFactor];
+    [aCoder encodeValuesOfObjCTypes:"cif", &thumbnail, &fileType , &compressionFactor];
     [aCoder encodeValuesOfObjCTypes:"@", &sourcePath];
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
@@ -1826,7 +1947,7 @@ y = Max(0, y-20);*/
 
     if (version >= 3)
     {   [aDecoder decodeValuesOfObjCTypes:"@", &clipPath];
-        [aDecoder decodeValuesOfObjCTypes:"cif", &thumbnail, &compressionType , &compressionFactor];
+        [aDecoder decodeValuesOfObjCTypes:"cif", &thumbnail, &fileType , &compressionFactor];
         [aDecoder decodeValuesOfObjCTypes:"@", &sourcePath];
     }
     return self;
@@ -1849,13 +1970,14 @@ y = Max(0, y-20);*/
     if (clipPath)
         [plist setObject:[clipPath propertyList] forKey:@"clipPath"];
     if (thumbnail) [plist setObject:@"YES" forKey:@"thumbnail"];
-    [plist setInt:compressionType forKey:@"compressionType"];
-    [plist setObject:propertyListFromFloat(compressionFactor) forKey:@"compressionFactor"];
+    [plist setInt:fileType forKey:@"fileType"];
+    if (fileType == VImageJPEG)
+        [plist setObject:propertyListFromFloat(compressionFactor) forKey:@"compressionFactor"];
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
 {   id		plistObject, obj;
-    NSString	*className, *str;
+    NSString	*className, *str, *path;
 
     [self setParameter];
     [super initFromPropertyList:plist inDirectory:directory];
@@ -1866,32 +1988,41 @@ y = Max(0, y-20);*/
     if (identifier >= currentGraphicIdentifier)
         currentGraphicIdentifier = identifier+1;
     imageFile = [[plist objectForKey:@"imageFile"] retain];
+
+    /* thumbnail */
     thumbnail = ([plist objectForKey:@"thumbnail"] ? YES : NO);
     if (thumbnail)
-        str = [NSString stringWithFormat:@"%@_.%@", [imageFile stringByDeletingPathExtension], [imageFile pathExtension]];
+        str = [NSString stringWithFormat:@"%@_.%@", [imageFile stringByDeletingPathExtension],
+                                                    [imageFile pathExtension]];
     if (thumbnail &&
         [[NSFileManager defaultManager] fileExistsAtPath:[directory stringByAppendingPathComponent:str]])
     {   thumbImage = [[NSImage allocWithZone:(NSZone *)[self zone]] init];
-        [thumbImage addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[directory 			stringByAppendingPathComponent:str]]];
+        path = [directory stringByAppendingPathComponent:str];
+        [thumbImage addRepresentations:[NSImageRep imageRepsWithContentsOfFile:path]];
         [thumbImage setDataRetained:YES];
 	[thumbImage setScalesWhenResized:YES];
     }
-    else if (![imageFile hasSuffix:@".tiff"])
-    {   image = [[NSUnarchiver unarchiveObjectWithFile:[directory stringByAppendingPathComponent:imageFile]] retain];
-        [imageFile release]; // war autorelease
+    // FIXME: we should only convert to TIFF, if other formats are not available !
+    else if (![imageFile hasSuffix:@".tiff"])	// convert to tiff
+    {
+        path = [directory stringByAppendingPathComponent:imageFile];
+        image = [[NSUnarchiver unarchiveObjectWithFile:path] retain];
+        [imageFile release];	// war autorelease
         imageFile = [[imageFile stringByAppendingString:@".tiff"] retain];
     }
     else
     {   image = [[NSImage allocWithZone:(NSZone *)[self zone]] init];
-        [image addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[directory	stringByAppendingPathComponent:imageFile]]];
+        path = [directory stringByAppendingPathComponent:imageFile];
+        [image addRepresentations:[NSImageRep imageRepsWithContentsOfFile:path]];
         [image setDataRetained:YES];
 	[image setScalesWhenResized:YES];
     }
     sourcePath = [[directory stringByAppendingPathComponent:imageFile] retain];
-    originalSize = [((thumbImage)?thumbImage:image) size];
+    originalSize = [((thumbImage) ? thumbImage : image) size];
 
     relief = ([plist objectForKey:@"relief"] ? YES : NO);
     reliefType = [plist intForKey:@"reliefType"];
+
     /* clipPath */
     plistObject = [plist objectForKey:@"clipPath"];
     if (plistObject)
@@ -1901,12 +2032,17 @@ y = Max(0, y-20);*/
             obj = [NSClassFromString(newClassName(className)) allocWithZone:[self zone]];
         clipPath = [obj initFromPropertyList:plistObject inDirectory:directory];
     }
-    else clipPath = nil;
+    else
+        clipPath = nil;
 
-    compressionType = [plist intForKey:@"compressionType"];
-    if (!compressionType)
-        compressionType = NSTIFFCompressionNone;
+    if ([plist objectForKey:@"compressionType"])	// old (< 3.64 2004-10-06)
+        fileType = fileTypeForTIFFCompression([plist intForKey:@"compressionType"]);
+    else
+        fileType = [plist intForKey:@"fileType"];
+    if (!fileType)
+        fileType = VImageTIFF_None;	// default
     compressionFactor = [plist floatForKey:@"compressionFactor"];
+
     return self;
 }
 
